#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set et sw=4 fenc=utf-8:
#
# Copyright 2016~2018 INVITE Communications Co., Ltd. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

""" ivr - A set of IVR functions for Asterisk.

The pyst project includes several python modules to assist in programming
asterisk with python:

ivr     - The IVR Template

"""

#from ivr.ivr import *

__all__ = ['ivr','test','connection', 'odbc']
__version__ = '0.0.9'

#from .test import joke



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set et sw=4 fenc=utf-8:
#
# Copyright 2016~2018 INVITE Communications Co., Ltd. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Asterisk Configuration File
######################################################
This module is used to read the database settings from an Asterisk configuration file.
It uses the Python ConfigParser to load the setting values directly from the specified
section (context) of the Asterisk configuration file.

.. note:: You need the `MySQL Connector/Python <https://dev.mysql.com/downloads/connector/python/>`_ installed on your system to connect to the database.

Example:
    Import the module, get the config settings, and pass them to your mysql connection::

        import ivr.connection
        
        config = ivr.connection.config()
         -or-
        config = ivr.connection.config('context')
        
        database = mysql.connect(**config)

"""

asterisk_path = '/etc/asterisk'
"""String: Asterisk Configuration Directory Path

The path to the Asterisk Configuration Files for **your** installation.
"""

asterisk_conf = 'res_config_mysql.conf'
"""String: Name of the configuration file you want to use.

.. warning:: 

    The MySQL Connector/Python does not support DSN configuration options.
    
    **Do not use Asterisk ODBC configuration files!**
    
You can use any Asterisk configuration file that contains the host, database, user, and password details.  
*You could create a dedicate configuration file too.*
"""

context = 'general'
"""String: The section (context) of the configuration file to use.

The Asterisk configuration file can be read by the Python ConfigParser.
Just specify the section of the configuration you would like to use.
The section is contained in square brackets, as shown in the following configuration file example::

    [general]
    dbhost = 127.0.0.1
    dbname = asterisk
    dbuser = myuser
    dbpass = mypass
    dbport = 3306
    dbsock = /tmp/mysql.sock
    dbcharset = utf8
    requirements=warn  ; or createclose or createchar        
"""

import os
import ConfigParser

settings = ConfigParser.RawConfigParser()
settings.read(os.path.join(asterisk_path, asterisk_conf))


def config(*args):
    if not args:
        data = {
            'user':              settings.get(context, 'dbuser'),
            'password':          settings.get(context, 'dbpass'),
            'host':              settings.get(context, 'dbhost'),
            'database':          settings.get(context, 'dbname'),
            'raise_on_warnings': True,
        }
        return data
    else:
        data = {
            'user':              settings.get(args[0], 'dbuser'),
            'password':          settings.get(args[0], 'dbpass'),
            'host':              settings.get(args[0], 'dbhost'),
            'database':          settings.get(args[0], 'dbname'),
            'raise_on_warnings': True,
        }
        return data



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set et sw=4 fenc=utf-8:
#
# Copyright 2016~2018 INVITE Communications Co., Ltd. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
    module: ivr

"""

import ConfigParser
#from test import joke


class IVR:
    """Example function with types documented in the docstring.

    `PEP 484`_ type annotations are supported. If attribute, parameter, and
    return types are annotated according to `PEP 484`_, they do not need to be
    included in the docstring:

    Args:
        param1 (int): The first parameter.
        param2 (str): The second parameter.

    Returns:
        bool: The return value. True for success, False otherwise.

    .. _PEP 484:
        https://www.python.org/dev/peps/pep-0484/

    Examples:
        Examples should be written in doctest format, and should illustrate how
        to use the function.

        >>> print([i for i in example_generator(4)])
        [0, 1, 2, 3]

    """
    
    def __init__(self):
        """Load the configuration.
        """
        #       	settings = ConfigParser.RawConfigParser()
        #        settings.read(config)
        #        self.foo = settings.get('metadata', 'description-file')
		
    def f(self):
        """
        agi.say_time(seconds, escape_digits='') --> digit
        Say a given time, returning early if any of the given DTMF digits are
        pressed.  
        The time should be in seconds since the UNIX Epoch (Jan 1, 1970 00:00:00)
        
        Example:
            This is what an example should look like.
            
            for 10 in 20 {
                // Do something
            }
        """
        return self.foo

    def a(self, text):
        """
        Is this what we all want to see?
        """
        return text
    
    def dummy(self):
        return 'nothing to see here'
        """ Testing Read The Docs Content Generation """

    def dtmf(file, valid_digits):
        regexp = re.compile(r'[' + valid_digits + ']')
    
        res = agi.get_data(file, 2000, 1)
        if regexp.search(res) is not None:
            return res

        res = agi.get_data(file, 2000, 1)    
        if regexp.search(res) is not None:
            return res
        
        if not res:
            agi.hangup()        
        
        



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set et sw=4 fenc=utf-8:
#
# Copyright 2016 INVITE Communications Co., Ltd. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


"""Packaging files and information."""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from ivr import __version__ as version

setup(
    name = 'ivr',
    packages = ['ivr'], # this must be the same as the name above
    version = version,
    description = 'AGI Controlled IVR for Asterisk',
    author = 'Brian LaVallee',
    author_email = 'brian.lavallee@invite-comm.jp',
    url = 'https://github.com/invitecomm/asterisk-ivr', # use the URL to the github repo
    download_url = 'https://github.com/invitecomm/asterisk-ivr/tarball/' + version, # I'll explain this in a second
    keywords = 'python asterisk agi ivr telephony telephony sip voip',
    classifiers = [
        #'Development Status :: 1 - Planning',
        'Development Status :: 2 - Pre-Alpha',
    ],
    license = 'GNU General Public License',

    # Package dependencies:
    install_requires = [
        'six>=1.9.0',
        'google-api-python-client==1.5.3',
        'pyst2',
        'mysql-connector-python>=2.1.0',
        'retry',
        'redis==2.10.*'
    ],
    
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set et sw=4 fenc=utf-8:
#
# Copyright 2016~2018 INVITE Communications Co., Ltd. All Rights Reserved.
# Copyright 2018 Sergei Turukin All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Asterisk Configuration File
######################################################
This module is used to read the database settings from an Asterisk configuration file.  It uses the Python ConfigParser to load the setting values directly from the specified section (context) of the Asterisk configuration file.

.. note:: You need the `MySQL Connector/Python <https://dev.mysql.com/downloads/connector/python/>`_ installed on your system to connect to the database.

Example:
    Import the module, get the config settings, and pass them to your mysql connection::

        import ivr.connection

        config = ivr.connection.config()
         -or-
        config = ivr.connection.config('context')

        database = mysql.connect(**config)

"""

asterisk_path = '/etc/asterisk'
"""String: Asterisk Configuration Directory Path

The path to the Asterisk Configuration Files for **your** installation.
"""

asterisk_conf = 'res_config_mysql.conf'
"""String: Name of the configuration file you want to use.

.. warning::

    The MySQL Connector/Python does not support DSN configuration options.

    **Do not use Asterisk ODBC configuration files!**

You can use any Asterisk configration file that contains the host, database, user, and password details.
*You could create a dedicated configuration file too.*
"""

context = 'general'
"""String: The section (context) of the configration file to use.

The Asterisk configuration file can be read by the Python ConfigParser.  Just specify the section of the configration you would like to use.  The section is contained in square brackets, as shown in the following configuration file example::

    [general]
    dbhost = 127.0.0.1
    dbname = asterisk
    dbuser = myuser
    dbpass = mypass
    dbport = 3306
    dbsock = /tmp/mysql.sock
    dbcharset = utf8
    requirements=warn ; or createclose or createchar

"""

odbc_context = 'asterisk'
"""String: The section (context) of the ODBC configration file to use.

The Asterisk configuration file can be read by the Python ConfigParser.  Just specify the section of the configration you would like to use.  The section is contained in square brackets, as shown in the following configuration file example::

[asterisk]
enabled => no
dsn => MySQL-asterisk
username => myuser
password => mypass
pre-connect => yes

"""

import os
import ConfigParser


RES_ODBC_CONF = os.path.join(asterisk_path, 'res_odbc.conf')
ODBC_INI = os.environ.get('ODBCINI', '/etc/odbc.ini')

odbc2mysql = {
    'user': 'user',
    'uid':  'user',
    'server': 'host',
    'database': 'database',
    'port': 'port',
    'password': 'password',
    'pwd': 'password',
    'socket': 'unix_socket',
    'sslca': 'ssl_ca',
    'sslcert': 'ssl_cert',
    'sslkey': 'ssl_key',
    'sslverify': 'ssl_verify_cert',
    'charset': 'charset',
}


def load_from(path):
    settings = ConfigParser.RawConfigParser()
    settings.read(path)
    return settings


def dsn_from(path, context='asterisk'):
    return safe_get_from(
        load_from(path), context, ['dsn']).replace('>', '').strip()


def creds_from(path, context='asterisk'):
    odbc_conf = load_from(path)
    user = safe_get_from(odbc_conf, context, ['username'])
    password = safe_get_from(odbc_conf, context, ['password'])
    return user.replace('>', '').strip(), password.replace('>', '').strip()

def discard_comments(value):
    if type(value) == str:
        return value if '#' not in value else value.split('#')[0].strip()
    else:
        return value

def safe_get_from(settings, section, keys):
    for key in keys:
        try:
            return settings.get(section, key)
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError) as err:
            # TODO: replace with logging
            print(err)
    return ''


def load_from_odbc(path, dsn=''):
    settings = load_from(path)

    driver = discard_comments(settings.get(dsn, 'driver'))
    if driver != 'MySQL':
        raise Exception("Expected MySQL driver, found %s", driver)

    items = {}
    for k, v in settings.items(dsn):
        if odbc2mysql.get(k, None):
            items[odbc2mysql[k]] = v

    # careful with iterating dict modification (at least, in py3)
    for k, v in items.items():
        items[k] = discard_comments(v)
    return items


def config_from_odbc(path, context):
    dsn = dsn_from(RES_ODBC_CONF, context=context)
    if not dsn:
        return {}
    return load_from_odbc(ODBC_INI, dsn)


def config(context=odbc_context, override_creds=False):
    data = config_from_odbc(ODBC_INI, context)
    if override_creds:
        data['user'], data['password'] = creds_from(RES_ODBC_CONF, context=context)
    #remove_junk(data)
    return data


def remove_junk(items):
    connector_acceptable_fields = {
        'user',
        'password',
        'database',
        'host',
        'port',
        'unix_socket',
        'auth_plugin',
        'use_unicode',
        'charset',
        'collation',
        'autocommit',
        'time_zone',
        'sql_mode',
        'get_warnings',
        'raise_on_warnings',
        'connection_timeout',
        'client_flags',
        'buffered',
        'raw',
        'consume_results',
        'ssl_ca',
        'ssl_cert',
        'ssl_key',
        'ssl_verify_cert',
        'force_ipv6',
        'dsn',
        'pool_name',
        'pool_size',
        'pool_reset_session',
        'compress',
        'converter_class',
        'failover',
        'option_files',
        'option_groups',
        'allow_local_infile',
        'use_pure'
    }
    # Remove all that is not compatible with MySQL connector API
    # see https://dev.mysql.com/doc/connector-python/en/connector-python-connectargs.html
    data = dict((k, v) for k, v in items if k in connector_acceptable_fields)
    return data




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set et sw=4 fenc=utf-8:
"""
A Simple Example
"""

from asterisk.agi import *

myVar = 'some data'
"""
A Simple Variable
"""

agi = AGI()
"""
Create an AGI Instance 
"""

agi.answer()
"""
I/O Communications: stdin, stdout, and stderr
"""